package com.blazinggames.retro.modern;

// Java imports
import java.awt.*;

// Blazing Games imports
import com.blazinggames.urcl.ui.*;
import com.blazinggames.urcl.ui.comp.*;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * The applet class that handles a generic title screen. The title screen
 * makes use of the configuration file in order to get properties specific to
 * the title. In order to allow multiple title/menu screens in a single
 * program, a prefix value can be attached to the config values. The values are
 * <UL>
 * <LI>TITLE_FONT the font to use</LI>
 * <LI>TITLE_BTNCOLORNORM the buttons normal color</LI>
 * <LI>TITLE_BTNCOLOROVER the color of the button when mouse is over</LI>
 * <LI>TITLE_BTNCOLORDOWN the color of the button when the mouse is down</LI>
 * <LI>TITLE_BTNCOLORTEXT the color of the button text</LI>
 * <LI>TITLE_BACKDROP an image to use for the background</LI>
 * <LI>TITLE_NUMBUTTONS the number of buttons in the menu</LI>
 * <LI>TITLE_BUTTON an array of buttons with their location, name, and command</LI>
 * </UL> 
 * 
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class Title
extends Display
{
    // *********************
    // ***** CONSTANTS *****
    // *********************

    private static final String FONT = "TITLE_FONT";
    private static final String BUTTON_NORM_COLOR = "TITLE_BTNCOLORNORM";
    private static final String BUTTON_OVER_COLOR = "TITLE_BTNCOLOROVER";
    private static final String BUTTON_DOWN_COLOR = "TITLE_BTNCOLORDOWN";
    private static final String BUTTON_TEXT_COLOR = "TITLE_BTNCOLORTEXT";
    private static final String BACKDROP = "TITLE_BACKDROP";
    private static final String NUM_BUTTONS = "TITLE_NUMBUTTONS";
    private static final String BUTTON = "TITLE_BUTTON";


    // *********************
    // ***** VARIABLES *****
    // *********************

    protected String _preface;
    protected Image _background;
    protected TextButton _buttons[];
    protected String _buttonActions[];


// ***********************
// ***** CONSTRUCTOR *****
// ***********************

	/**
	 * creates a title using just the default config values
	 */
    public Title()
    {
        _preface = "";
    }

	/**
	 * creates a title using a prefix in front of the config values
	 * 
	 * @param pref prefix to use for config strings
	 */
    public Title(String pref)
    {
        _preface = pref;
    }


// **********************************
// ***** METHODS - From Display *****
// **********************************

	/**
	 * display initialization method.
	 */
    public boolean initialize(DisplayHandler dh)
    {
        super.initialize(dh);

        Font fnt = _config.getFontProperty(_preface + FONT, font);
        Color btnNorm = _config.getColorProperty(_preface + BUTTON_NORM_COLOR, Color.blue);
        Color btnOver = _config.getColorProperty(_preface + BUTTON_OVER_COLOR, Color.blue);
        Color btnDown = _config.getColorProperty(_preface + BUTTON_DOWN_COLOR, TextButton.DARK_BLUE);
        Color btnText = _config.getColorProperty(_preface + BUTTON_TEXT_COLOR, Color.black);
        _res.addImage(_preface + BACKDROP, _config.getStringProperty(_preface + BACKDROP, "title.jpg"));
        _background = _res.getImage(_preface + BACKDROP);
        int n = _config.getIntProperty(_preface + NUM_BUTTONS, 1);

        _buttons = new TextButton[n];
        _buttonActions = new String[n];
        int x, y, w, h;
        String args[];
        Container c = _displayHandler.getContainer();

        for (int cntr = 0; cntr < n; ++cntr)
        {
            args = _config.parseString(_config.getStringProperty(_preface + BUTTON +"["+cntr+"]", "unnamed,0,0,100,20,QUIT"),',');
            if (args.length != 6)
            {
                _buttons[cntr] = new TextButton(c, "Undefined", 0, 0, 100, 20);
                _buttonActions[cntr] = "QUIT";
            }
            else
            {
                _buttons[cntr] = new TextButton(c, args[0],
                                                _config.parseInt(args[1], 0), _config.parseInt(args[2], 0),
                                                _config.parseInt(args[3], 100), _config.parseInt(args[4], 20) );
                _buttonActions[cntr] = args[5];
            }
            _buttons[cntr].setNormalColors(btnNorm, btnText);
            _buttons[cntr].setOverColors(btnOver, btnText);
            _buttons[cntr].setSelectedColors(btnDown, btnText);
            _buttons[cntr].setFont(c, fnt);
        }

        return true;
    }

    /*
     public boolean begin()
     {
         _active = true;
         return true;
     }

     public boolean end()
     {
         _active = false;
         return false;
     }
     */

	/**
	 * display's mouse movement method overridden to handle buttons
	 */
    public void mouse_move(int x, int y, boolean b)
    {
        boolean repaint = false;

        for (int cntr = 0; cntr < _buttons.length; ++cntr)
        {
            repaint |= _buttons[cntr].mouse_move(x, y, b);
        }

        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }

	/**
	 * display's mouse_down method overriden to handle buttons
	 */
    public void mouse_down(int x, int y)
    {
        boolean repaint = false;

        for (int cntr = 0; cntr < _buttons.length; ++cntr)
        {
            repaint |= _buttons[cntr].mouse_down(x, y);
        }

        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }

	/**
	 * display funtion overridden so buttons can be supported.
	 */
    public void mouse_up(int x, int y)
    {
        boolean repaint = false;

        for (int cntr = 0; cntr < _buttons.length; ++cntr)
        {
            if (_buttons[cntr].mouse_up(x, y))
            {
                _displayHandler.displayEvent(this, _buttonActions[cntr]);
                repaint = true;
            }
        }

        if (repaint)
        {
            _displayHandler.refreshDisplay(this);
        }
    }

	/**
	 * display render function overriden to draw display
	 */
    public synchronized void render(Graphics g, Component comp)
    {
        if (_background == null)
        {
            Dimension d = comp.getSize();
            g.setColor(Color.black);
            g.fillRect(0,0, d.width, d.height);
        }
        else
        {
            g.drawImage(_background, 0, 0, comp);
        }

        for (int cntr = 0; cntr < _buttons.length; ++cntr)
        {
            _buttons[cntr].render(g, comp);
        }
    }

}
